/**
 * Copyright 2018 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
interface TaskQueueOptions extends WorkerOptions {
    size?: number;
}
declare type TaskId = number;
declare type WorkerId = number;
declare const enum Status {
    RESOLVE = 0,
    REJECT = 1
}
declare type TaskResultDesc = [TaskId, Status, any?];
interface WorkerTaskQueue {
    $$pool: TaskQueuePool;
}
declare class WorkerTaskQueue {
    constructor(workerUrl: string, options?: TaskQueueOptions);
    postTask<T>(taskName: string, ...args: any[]): Task<T>;
    destroy(): void;
}
export default WorkerTaskQueue;
interface TaskWorker extends Worker {
    id: WorkerId;
    pending: number;
    ready: Promise<void>;
    call<T = any>(method: string, args: any[]): Promise<T>;
}
declare type PromiseController<T = any, E = any> = {
    [Status.RESOLVE]: (value: T) => void;
    [Status.REJECT]: (error: E) => void;
};
interface ResultController<T = any> extends PromiseController {
    /** is the task waiting to be sent to a worker? */
    pending: boolean;
    /** has the task been cancelled? */
    cancelled: boolean;
    completed: boolean;
    fulfilled: boolean;
    requested: boolean;
    result: Promise<T>;
}
interface TaskQueuePool {
    poolSize: number;
    workerUrl: string;
    workerOptions?: WorkerOptions;
    workers: TaskWorker[];
    tasks: Record<number, Task<any>>;
    results: Record<number, ResultController>;
    workerTaskAssignments: Record<TaskId, WorkerId>;
}
declare class TaskQueuePool {
    constructor(poolSize: number, workerUrl: string, workerOptions?: WorkerOptions);
    destroy(): void;
    exec<T = any>(task: Task<T>, taskName: string, args: any[]): void;
    /**
     * Cancel a task by its ID.
     * Cancellation is not guaranteed, since task may already have started executing.
     * Cancelling an already-completed task returns `false`.
     */
    cancel(taskId: number): boolean | void;
    /**
     * Returns a Promise that will be fulfilled with the Task's value/error.
     */
    getResult(taskId: number): Promise<any>;
    freeWorkerTask(worker: TaskWorker): void;
    statusReceived(worker: TaskWorker, statuses: TaskResultDesc[]): void;
    addWorker(): TaskWorker;
    getWorker(id: WorkerId): TaskWorker;
    getWorkerForTask(taskId: TaskId): TaskWorker;
    getTaskDependencies(args: any[]): Task<any>[];
    getTaskWorker(taskName: string, args: any[]): TaskWorker;
    getNextWorker(): TaskWorker;
}
interface Task<T> {
    id: number;
    state: string;
    result: T;
    $$taskIdentifier: string;
    $$result: Promise<T>;
    $$queue: WorkerTaskQueue;
}
declare class Task<T> {
    cancel(): void;
}
