/**
 * Copyright 2018 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

export type Key = string | number;

type Reducer<T> = (
  accumulator: Partial<T> | void,
  obj: any,
  index: Key,
  parent: any
) => Partial<T> | void;

export function walkReduce<T = any>(
  obj: any,
  reducer: Reducer<T>,
  accumulator?: Partial<T> | void,
  index?: Key,
  parent?: any
) {
  let result = reducer(accumulator, obj, index, parent);
  if (result === undefined) result = accumulator;
  if (typeof obj === 'object' && obj) {
    for (let i in obj) {
      walkReduce<T>(obj[i], reducer, result, i, obj);
    }
  }
  return result;
}

type Transferrables = ArrayBuffer | MessagePort | ImageBitmap;

// This is done to support Node, which lacks ImageBitmap and sometimes MessagePort:
function Never() {}
const MessagePortConstructor = (typeof MessagePort === 'undefined' ? Never : MessagePort) as any as typeof MessagePort;
const ImageBitmapConstructor = (typeof ImageBitmap === 'undefined' ? Never : ImageBitmap) as any as typeof ImageBitmap;

export function collectTransferrables<T = any>(xfer: Transferrables[], value: T) {
  if (
    value instanceof ArrayBuffer ||
    value instanceof MessagePortConstructor ||
    value instanceof ImageBitmapConstructor
  ) {
    xfer.push(value);
  }
}
